//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  SocketCAN header translation by Zeljko Avramovic                                    //
//  More info can be found at https://www.kernel.org/doc/html/v5.10/networking/can.html //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

unit can.j1939;

{$packrecords c}

interface

uses
  ctypes, can;


(* SPDX-License-Identifier: GPL-2.0-only WITH Linux-syscall-note *)
(*
 * j1939.h
 *
 * Copyright (c) 2010-2011 EIA Electronics
 *
 * This aProgram is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *)

const
  J1939_MAX_UNICAST_ADDR      = $fd;
  J1939_IDLE_ADDR             = $fe;
  J1939_NO_ADDR               = $ff;     (* == broadcast or no addr *)
  J1939_NO_NAME               = 0;
  J1939_PGN_REQUEST           = $0ea00;  (* Request PG              *)
  J1939_PGN_ADDRESS_CLAIMED   = $0ee00;  (* Address Claimed         *)
  J1939_PGN_ADDRESS_COMMANDED = $0fed8;  (* Commanded Address       *)
  J1939_PGN_PDU1_MAX          = $3ff00;
  J1939_PGN_MAX               = $3ffff;
  J1939_NO_PGN                = $40000;

type
  (* J1939 Parameter Group Number
   *
   * bit 0-7   : PDU Specific (PS)
   * bit 8-15  : PDU Format (PF)
   * bit 16    : Data Page (DP)
   * bit 17    : Reserved (R)
   * bit 19-31 : set to zero
   *)
  pgn_t = cuint32;

  (* J1939 Priority
   *
   * bit 0-2   : Priority (P)
   * bit 3-7   : set to zero
   *)
  priority_t = cuint8;

  (* J1939 NAME
   *
   * bit 0-20  : Identity Number
   * bit 21-31 : Manufacturer Code
   * bit 32-34 : ECU Instance
   * bit 35-39 : aFunction Instance
   * bit 40-47 : aFunction
   * bit 48    : Reserved
   * bit 49-55 : Vehicle System
   * bit 56-59 : Vehicle System Instance
   * bit 60-62 : Industry Group
   * bit 63    : Arbitrary Address Capable
   *)
  name_t = cuint64;

const
  (* J1939 socket options *)
  SOL_CAN_J1939 = SOL_CAN_BASE + CAN_J1939;

  SO_J1939_FILTER        = 1;  (* set filters *)
  SO_J1939_PROMISC       = 2;  (* set/clr promiscuous mode *)
  SO_J1939_SEND_PRIO     = 3;
  SO_J1939_ERRQUEUE      = 4;

  SCM_J1939_DEST_ADDR    = 1;
  SCM_J1939_DEST_NAME    = 2;
  SCM_J1939_PRIO         = 3;
  SCM_J1939_ERRQUEUE     = 4;

  J1939_NLA_PAD          = 0;
  J1939_NLA_BYTES_ACKED  = 1;

  J1939_EE_INFO_NONE     = 0;
  J1939_EE_INFO_TX_ABORT = 1;

type
  j1939_filter = record
    name:      name_t;
    name_mask: name_t;
    pgn:       pgn_t;
    pgn_mask:  pgn_t;
    addr:      cuint8;
    addr_mask: cuint8;
  end;

const
  J1939_FILTER_MAX = 512;  (* maximum number of j1939_filter set via setsockopt() *)

implementation

end.

