//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This is a SocketCAN demo converted from C to Pascal by Zeljko Avramovic.    //
//  Original example has GNU General Public License v3.0 and can be found at    //
//  https://github.com/craigpeacock/CAN-Examples                                //
//                                                                              //
//  Virtual CAN adapter vcan0 is hard coded and you can bring it up like this:  //
//  sudo modprobe vcan                                                          //
//  sudo ip link add dev vcan0 type vcan                                        //
//  sudo ip link set vcan0 mtu 72            # needed for CAN FD                //
//  sudo ip link set vcan0 up                                                   //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

program canreceive;

{$mode delphi} {$H+}

uses
  classes, sysutils, sockets, baseunix, can;

procedure perror(const S : string);
begin
  WriteLn(S, ', SocketError = ', SocketError);
end;

var
  s, i, nbytes: cint;
  addr:         sockaddr_can;
  ifr:          ifreq;
  frame:        can_frame;
begin
  try
    WriteLn('CAN Sockets Receive Demo');
    s := fpsocket(PF_CAN, SOCK_RAW, CAN_RAW);
    if s < 0 then
    begin
      perror('Socket');
      Exit;
    end;

    strcopy(ifr.ifr_name, 'vcan0'); // ifr.ifr_name := 'vcan0' + #0;
    if fpioctl(s, SIOCGIFINDEX, @ifr) < 0 then
    begin
      perror('IOctl');
      Exit;
    end;

    memset(@addr, 0, sizeof(addr));
    addr.can_family  := AF_CAN;
    addr.can_ifindex := ifr.ifr_ifindex;

    if fpbind(s, @addr, sizeof(addr)) < 0 then // if fpbind(s, Psockaddr(@addr), sizeof(addr)) < 0 then
    begin
      perror('Bind');
      Exit;
    end;

    nbytes := fpread(s, frame, sizeof(can_frame));
    if nbytes < 0 then
    begin
      perror('Read');
      Exit;
    end;
    if nbytes < sizeof(can_frame) then // paranoid check
    begin
      perror('Read: Incomplete CAN frame');
      Exit;
    end;

    Write(Format('0x%03X [%d] ', [frame.can_id and CAN_SFF_MASK, frame.can_dlc]));
    for i := 0 to frame.can_dlc - 1 do
      Write(Format('%02.2X ', [frame.data[i]]));
    WriteLn('');

    if fpclose(s) < 0 then
    begin
      perror('Close');
      Exit;
    end;

  except
    on e: Exception do
      WriteLn(e.Message);
  end;
end.

