//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This is a SocketCAN demo converted from C to Pascal by Zeljko Avramovic.    //
//  Original example has GNU General Public License v3.0 and can be found at    //
//  https://github.com/craigpeacock/CAN-Examples                                //
//                                                                              //
//  Virtual CAN adapter vcan0 is hard coded and you can bring it up like this:  //
//  sudo modprobe vcan                                                          //
//  sudo ip link add dev vcan0 type vcan                                        //
//  sudo ip link set vcan0 mtu 72            # needed for CAN FD                //
//  sudo ip link set vcan0 up                                                   //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

program cantransmit;

{$mode delphi} {$H+}

uses
  classes, sysutils, sockets, baseunix, can;

procedure perror(const S : string);
begin
  WriteLn(S, ', SocketError = ', SocketError);
end;

var
  s:     cint;
  addr:  sockaddr_can;
  ifr:   ifreq;
  frame: can_frame;
begin
  try
    WriteLn('CAN Sockets Transmit Demo');
    s := fpsocket(PF_CAN, SOCK_RAW, CAN_RAW);
    if s < 0 then
    begin
      perror('Socket');
      Exit;
    end;

    strcopy(ifr.ifr_name, 'vcan0'); // ifr.ifr_name := 'vcan0' + #0;
    if fpioctl(s, SIOCGIFINDEX, @ifr) < 0 then
    begin
      perror('IOctl');
      Exit;
    end;

    memset(@addr, 0, sizeof(addr));
    addr.can_family  := AF_CAN;
    addr.can_ifindex := ifr.ifr_ifindex;

    if fpbind(s, @addr, sizeof(addr)) < 0 then // if fpbind(s, Psockaddr(@addr), sizeof(addr)) < 0 then
    begin
      perror('Bind');
      Exit;
    end;

    frame.can_id   := $555;    // CAN message ID
    frame.can_dlc  := 5;       // payload length will be 5 bytes
    frame.DataText := 'Hello'; // datatext field is a variant record of chars overlaying data field for easy string messages, shows as 0x555 [5] 48 65 6C 6C 6F
    // frame.data     := [$11, $22, $33, $44, $55, $00, $00, $00]; // if you want to send binary data instead of text, shows as 0x555 [5] 11 22 33 44 55
    // frame.data[0]  := $11; // alternative for the above, shows as 0x555 [5] 11 22 33 44 55 in canreceive
    // frame.data[1]  := $22;
    // frame.data[2]  := $33;
    // frame.data[3]  := $44;
    // frame.data[4]  := $55;
    // frame.DataQword[0] := $1122334455667788; // quite different from the above and shows in reverse order as 0x555 [5] 88 77 66 55 44 in canreceive

    if fpwrite(s, frame, sizeof(can_frame)) <> sizeof(can_frame) then
    begin
      perror('Write');
      Exit;
    end;

    if fpclose(s) < 0 then
    begin
      perror('Close');
      Exit;
    end;

  except
    on e: Exception do
      WriteLn(e.Message);
  end;
end.

