//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This is a SocketCAN demo converted from C to Pascal by Zeljko Avramovic.    //
//  Original example has GNU General Public License v3.0 and can be found at    //
//  https://github.com/craigpeacock/CAN-Examples                                //
//                                                                              //
//  Virtual CAN adapter vcan0 is hard coded and you can bring it up like this:  //
//  sudo modprobe vcan                                                          //
//  sudo ip link add dev vcan0 type vcan                                        //
//  sudo ip link set vcan0 mtu 72            # needed for CAN FD                //
//  sudo ip link set vcan0 up                                                   //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

program hlcanreceive; // higher level pascalized version of canreceive demo

{$mode delphi} {$H+}

uses
  sysutils, sockets, can, can.hl;

procedure perror(const S : string);
begin
  WriteLn(S, ', SocketError = ', SocketError);
end;

var
  i, BytesNo:   integer;
  CanHandle:    TCanHandle;
  CanAddress:   TCanSockAddress;
  CanInterface: TCanIfreq;
  CanFrame:     TCanFrame;
begin
  try
    WriteLn('CAN Sockets Receive Demo');
    CanHandle := CanSocket(SOCK_RAW, CAN_RAW);
    if CanHandle < 0 then
    begin
      perror('Socket');
      Exit;
    end;

    CanInterface.Name := 'vcan0';
    if CanIOCtl(CanHandle, SIOCGIFINDEX, CanInterface) < 0 then
    begin
      perror('IOctl');
      Exit;
    end;

    CanAddress.Clear;
    CanAddress.CanFamily  := AF_CAN;
    CanAddress.CanIfIndex := CanInterface.IfIndex;

    if CanBind(CanHandle, CanAddress) < 0 then
    begin
      perror('Bind');
      Exit;
    end;

    CanFrame.Clear;               // comment this whole section and
    CanFrame.ID.RTR      := true; // you will not be able to receive
    CanFrame.ID.EFF      := true; // both SFF and EFF messages - instead
    CanFrame.ID.Mask.EFF := true; // you will receive only SFF messages

    BytesNo := CanRead(CanHandle, CanFrame);
    if BytesNo < 0 then
    begin
      perror('Read: Error = ' + CanGetErrNo.ToString);
      Exit;
    end;
    if BytesNo < SizeOf(CanFrame) then // paranoid check
    begin
      perror('Read: Incomplete CAN frame');
      Exit;
    end;

    Write(format('0x%-08X [%d] ', [CanFrame.ID.Value, CanFrame.DLC])); // enough space to print both SFF and EFF message IDs
    for i := 0 to CanFrame.DLC - 1 do
      Write(Format('%02.2X ', [CanFrame.Data[i]]));
    WriteLn('');

    if CanClose(CanHandle) < 0 then
    begin
      perror('Close');
      Exit;
    end;

  except
    on e: Exception do
      WriteLn(e.Message);
  end;
end.

